import os
import sys
import time
import re
import json
import asyncio
import logging
import requests
from threading import Thread
from django.conf import settings
import psycopg2
from pymemcache.client.base import Client

BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.append(BASE_DIR)

if os.environ.get('DJANGO_SETTINGS_MODULE') is None:
    os.environ['DJANGO_SETTINGS_MODULE'] = "cabinet.settings_extra"

# logging
log_place = '/var/log/uwsgi/cabinet/ipinfo2memcache.log'

# Key alive time
mem_time = 86400

# Postgres connect
DB_NAME = settings.DB_NAME_CONF
DB_USER = settings.DB_USER_CONF
DB_PASS = settings.DB_PASS_CONF
DB_HOST = settings.DB_HOST_CONF
try:
    DB_PORT = settings.DB_PORT_CONF
    if DB_PORT == '':
        DB_PORT = '5432'
except NameError:
    DB_PORT = '5432'

vGeoIP = settings.V_GEOIP

# Proxy settings
try:
    http_proxy = settings.HTTP_PROXY_CONF
    https_proxy = settings.HTTP_PROXY_CONF
    proxyDict = {
        "http": http_proxy,
        "https": https_proxy
    }
except AttributeError as error:
    proxyDict = {
        "http": None,
        "https": None
    }

user_agent = {'User-agent': settings.USER_AGENT}

url_geoip = settings.URI_GEOIP

cache_ip, cache_port = settings.CACHES['default']['LOCATION'].split(':')
cache_address = (cache_ip, int(cache_port))

mcache = Client(cache_address)

result = []


def local(ip):
    reg = re.compile(
        r"(^127\.)|(^10\.)|(^172\.1[6-9]\.)|(^172\.2\d\.)|"
        r"(^172\.3[0-1]\.)|(^192\.168\.)|(^100\.127\.49\.)|"
        r"(^100\.(6[4-9]|[7-9]\d|1[01]\d|12[0-7]))")
    if reg.match(ip):
        return True
    else:
        return False


def multicast(ip):
    reg = re.compile(
        r"(^22[4-9]\.)|(^23\d\.)")
    if reg.match(ip):
        return True

    if ip == '::1':
        return True

    return False


def chunks(li, n):
    for i in range(0, len(li), n):
        yield li[i:i + n]


def join_ip_list(ip_list):
    return '|'.join(x for x in ip_list)


def send_request(url):
    try:
        response = requests.get(
            url,
            proxies=proxyDict,
            headers=user_agent,
            timeout=settings.URI_GEOIP_TIMEOUT)
    except requests.exceptions.Timeout as e:
        response = None
        logging.error(str(e) + '  ' + type(e).__name__)
    except requests.exceptions.ProxyError as e:
        response = None
        logging.error(str(e) + '  ' + type(e).__name__)
    except Exception as e:
        logging.error(str(e) + '  ' + type(e).__name__)
        response = None
    return response


def split_request(text):
    text = text.split('<br>')
    if len(text) > 1:
        return text[0:-1]
    else:
        return text


def check_ip_from_memcached(ip_list):
    ips = []
    for ip in ip_list:
        try:
            get_ip = mcache.get(ip + vGeoIP)
        except Exception:
            get_ip = None
        if get_ip is None:
            if local(ip) is False:
                if multicast(ip) is False:
                    ips.append(ip)
        else:
            if json.loads(get_ip.decode('utf-8').replace("'", '"')).get('city') == '':
                ips.append(ip)
    return ips


def set_ip_to_memcached(ip_list):
    for q in ip_list:
        if "Enter valid IP" not in q:
            try:
                b = json.loads(q)
                if b.get('status', None) == 'success' \
                        or (b.get('status', None) == 'fail' and b.get('description', None) == 'Private IP'):
                    if mcache.get(b['ipaddress'] + vGeoIP) is None:
                        mcache.set(b['ipaddress'] + vGeoIP, json.dumps(b), 86400)
            except Exception as e:
                logging.error(str(e))


def get_ip_info(chunk, res):
    # logging.debug('Sending request to GeoIP')
    r = send_request(url_geoip + chunk[0])
    if r:
        # logging.debug('Response are given {}'.format(r.text))
        ip_list = split_request(r.text)
        if 'Private IP' in str(ip_list):
            ip_list = json.loads(ip_list)
            ip_list['ipaddress'] = chunk[0]
            ip_list = json.dumps(ip_list)

        res.append(ip_list)


def ip_memcache(loop):
    try:
        connect = psycopg2.connect(database=DB_NAME, user=DB_USER, password=DB_PASS, host=DB_HOST, port=DB_PORT)
        cursor = connect.cursor()
        cursor.execute("SELECT DISTINCT ip, timestamp FROM attack ORDER BY timestamp DESC LIMIT 1000;")
        ips_db = [item[0] for item in cursor.fetchall()]
        ips = check_ip_from_memcached(ips_db)
        chunked_list = list(chunks(ips, 5))
        thread_chunks = list(chunks(chunked_list, 10))
        for chunk in thread_chunks:
            # add delay
            start_time = time.time()
            multithreading(get_ip_info, chunk)
            diff = time.time() - start_time
            if diff < 1:
                time.sleep(diff)
            for res in result:
                set_ip_to_memcached(res)
            result.clear()
    except Exception as e:
        logging.error(str(e))

    try:
        connect = psycopg2.connect(database=DB_NAME, user=DB_USER, password=DB_PASS, host=DB_HOST, port=DB_PORT)
        cursor = connect.cursor()
        cursor.execute("SELECT DISTINCT ip, timestamp FROM mt_main ORDER BY timestamp DESC LIMIT 1000;")
        ips_db = [item[0] for item in cursor.fetchall()]
        ips = check_ip_from_memcached(ips_db)
        chunked_list = list(chunks(ips, 5))
        thread_chunks = list(chunks(chunked_list, 10))
        for chunk in thread_chunks:
            # add delay
            start_time = time.time()
            multithreading(get_ip_info, chunk)
            diff = time.time() - start_time
            if diff < 1:
                time.sleep(diff)
            for res in result:
                set_ip_to_memcached(res)
            result.clear()
    except Exception as e:
        logging.error(str(e))

    loop.call_later(11, ip_memcache, loop)


def multithreading(fun, urls):
    threads = []
    for ii in urls:
        thread = Thread(target=fun, args=[ii, result])
        thread.start()
        threads.append(thread)
    for thread in threads:
        thread.join()


# Event Loop
def main():
    logging.basicConfig(
        filename=log_place,
        level=logging.INFO,
        format='%(asctime)s %(levelname)s: %(message)s',
        datefmt='%d/%b/%Y:%H:%M:%S %z')
    logging.info('Started')
    loop = asyncio.get_event_loop()
    try:
        loop.call_soon(ip_memcache, loop)
        loop.run_forever()
    finally:
        loop.close()
        logging.info('Finished')


if __name__ == '__main__':
    main()
