import json
import logging
import multiprocessing

from datetime import datetime, timedelta, date
from pymemcache.client.base import Client
from django.conf import settings
from django.db import connections
from django.db.models import Count, DateTimeField
from django.db.models import Q
from main.models.logs import Logs
from main.models.wafuser import WAFUSER
from django.db.models.functions import Trunc
from main.common import get_location_by_ip, get_filtered_logs, attack_short_count, get_location_from_remote, chunks
from django.utils.translation import gettext as _
from main.search import search_filter
from main.utils import is_local, convert_from_punycode, convert_to_punycode

try:
    from uwsgidecorators import spool

    spoolIsrun = True
except Exception:
    spoolIsrun = False


    def spool(func):
        def func_wrapper(**arguments):
            return func(arguments)

        return func_wrapper

logger = logging.getLogger(__name__)

vAPP = settings.V_APP

cache_ip, cache_port = settings.CACHES['default']['LOCATION'].split(':')
cache_address = (cache_ip, int(cache_port))


@spool
def gen_charts(args):
    if spoolIsrun:
        connections.close_all()
    user_id = int(args['user_id'])
    req_get = json.loads(args['req_get'])
    new_end_date = args['end_date']
    get_start_date = req_get.get('start')
    get_end_date = req_get.get('end')
    start_date = datetime.strptime(get_start_date, '%d.%m.%Y').strftime('%Y-%m-%d')
    end_date_chart = args['end_date']
    end_date = datetime.strptime(get_end_date, '%d.%m.%Y').strftime('%Y-%m-%d')
    state = req_get.get('state', '').strip()
    s = req_get.get('search', '').strip()
    mcache = Client(cache_address)
    semaphore_mcache_key = str(user_id) + 'summary' + vAPP
    mcache_key = str(str(user_id) + '|' + str(end_date).replace(' ', '') + '|' +
                     str(start_date).replace(' ', '') + '|' + convert_to_punycode(s) +
                     '|' + state + '|' + vAPP).replace(' ', '_')

    ts = datetime.timestamp(datetime.now())
    mcache.set(semaphore_mcache_key, ts)
    try:
        # domain cookie
        q = req_get.get('domain')
        if q == 'None':
            q = None
        # waf id
        wafuser = WAFUSER.objects.get(id=user_id)
        user_alias = wafuser.waf_alias
        # waf domain
        domain_from_wafuser = wafuser.domain

        search_history = wafuser.search_history
        try:
            if search_history is None:
                search_history = ''
            search_history_dict = json.loads(search_history)
        except json.JSONDecodeError:
            search_history_dict = dict()
            for i in search_history.split(','):
                search_history_dict[i.strip()] = 1

        if s is not None and s != '':
            if 'charts:' + s in search_history_dict.keys():
                reb = dict()
                reb['charts:' + s] = search_history_dict['charts:' + s] + 1
                search_history_dict.pop('charts:' + s)
                reb.update(search_history_dict)
                search_history_dict = reb

            else:
                reb = dict()
                reb['charts:' + s] = 1
                reb.update(search_history_dict)
                search_history_dict = reb
        else:
            for el in search_history_dict.keys():
                if 'charts:' in el:
                    s = el.replace('charts:', '')
                    break
        if s == '#clean':
            s = ''

        while len(search_history_dict) > 1000:
            search_history_dict.popitem()

        wafuser.search_history = json.dumps(search_history_dict)
        wafuser.save()

        charts_history = []

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        sortbyfreq = True
        if sortbyfreq:
            tmp = dict()
            last_use = ''
            for el in list(search_history_dict.keys()):
                if 'charts:' in el and 'charts:' + s != el:
                    last_use = el
                    break

            if last_use in search_history_dict:
                tmp[last_use] = search_history_dict[last_use] if last_use != '' else '#clear'
                search_history_dict.pop(last_use)
            tmp.update({k: v for k, v in sorted(search_history_dict.items(), key=lambda item: item[1], reverse=True)})
            search_history_dict = tmp

        for el in search_history_dict.keys():
            if 'charts:' in el and '#clean' not in el and s.lower() in el.replace('charts:', '').lower():
                charts_history.append(el.replace('charts:', ''))

        charts_history_json = json.dumps(charts_history)

        today_date = date.today()
        formatted_summary_end_date = datetime.strptime(new_end_date, '%Y-%m-%d %H:%M:%S.%f').date()
        if formatted_summary_end_date > today_date:
            new_end_date = str(today_date) + ' 23:59:59.000000'

        piechart_end_date = datetime.strptime(new_end_date, '%Y-%m-%d %H:%M:%S.%f').date()
        yesterday_date = piechart_end_date - timedelta(days=1)
        last_week = piechart_end_date - timedelta(days=7)
        last_month = piechart_end_date - timedelta(days=30)

        main = Logs.objects.using('waf'). \
            only('vhost', 'alias', 'timestamp').filter(timestamp__gte=last_month, timestamp__lte=end_date_chart). \
            exclude(possible=True)

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        main = get_filtered_logs(main, user_alias, domain_from_wafuser, q)

        charts = main.values('vhost', 'alias', 'timestamp').filter(timestamp__gte=start_date,
                                                                   timestamp__lte=end_date_chart). \
            exclude(possible=True).exclude(bt=5)
        # If the state of the checkbox on the chart has changed Top by IP
        if state:

            s = s.strip()
            if s:
                try:
                    charts = search_filter(s, charts)
                except Exception:
                    charts = Logs.objects.using('waf').none()

            summary_end_date = new_end_date
            today_date = date.today()
            formatted_summary_end_date = datetime.strptime(summary_end_date, '%Y-%m-%d %H:%M:%S.%f').date()
            if formatted_summary_end_date > today_date:
                summary_end_date = str(today_date) + ' 23:59:59.000000'
            get_end_date = datetime.strptime(summary_end_date, '%Y-%m-%d %H:%M:%S.%f').date()
            yesterday_date = get_end_date - timedelta(days=1)
            last24 = charts.filter(timestamp__gte=yesterday_date, timestamp__lte=summary_end_date)

            last24h_ip_count_dict = dict()

            last24 = last24.values('ip').annotate(
                all_attacks=Count('bt'),
            ).order_by('-all_attacks')
            for i in last24:
                last24h_ip_count_dict[i.get('ip')] = str(i.get('all_attacks'))

            # if not all checkboxs active
            split_state = []
            if state != 'clear':
                split_state = state.split('-')
            exclude_bt = []
            for i in split_state:
                if i == 'Signature Analysis':
                    exclude_bt.append(1)
                    exclude_bt.append(2)
                if i == 'Nemesida MLA':
                    exclude_bt.append(3)
                if i == 'Bruteforce':
                    exclude_bt.append(7)
                if i == 'ClamAV':
                    exclude_bt.append(4)
                if i == 'Nemesida MLC':
                    exclude_bt.append(8)
                if i == 'flood':
                    exclude_bt.append(9)
                if i == 'Ddos':
                    exclude_bt.append(10)
            ip_info_data = charts.values('ip').filter(
                bt__in=[1, 2, 3, 4, 7, 8, 9, 10]).exclude(Q(bt__in=exclude_bt)).annotate(
                all_attacks=Count('bt'),
                bt1=Count('bt', filter=Q(bt=1)),
                bt2=Count('bt', filter=Q(bt=2)),
                bt3=Count('bt', filter=Q(bt=3)),
                bt4=Count('bt', filter=Q(bt=4)),
                bt7=Count('bt', filter=Q(bt=7)),
                bt8=Count('bt', filter=Q(bt=8)),
                bt9=Count('bt', filter=Q(bt=9)),
                bt10=Count('bt', filter=Q(bt=10)),
            ).order_by('-all_attacks')[:15]
            ip_info_row = list()

            ips_list2 = []
            for i in ip_info_data:
                ips_list2.append(i.get('ip'))

            processes = []

            lock = multiprocessing.Lock()
            for ip in list(chunks(list(set(ips_list2)), 1))[:len(ips_list2)]:
                proc = multiprocessing.Process(target=get_location_from_remote, args=(ip, mcache, lock))
                proc.start()
                processes.append(proc)

            for pr in processes:
                pr.join(5)

            for i in ip_info_data:

                if get_location_by_ip(mcache, i.get('ip')).get('status') != 'success':
                    location = get_location_by_ip(mcache, i.get('ip'), fail=False)
                else:
                    location = get_location_by_ip(mcache, i.get('ip'))

                if str(location['mobile']).lower() == 'none' or \
                        str(location['mobile']).lower() == 'false':
                    mobile = 'No'
                else:
                    mobile = 'Yes'

                if str(location['proxy']).lower() == 'none' or \
                        str(location['proxy']).lower() == 'false':
                    proxy = 'No'
                else:
                    proxy = 'Yes'

                if str(location['hosting']).lower() == 'none' or \
                        str(location['hosting']).lower() == 'false':
                    hosting = 'No'
                else:
                    hosting = 'Yes'

                if str(location['crawler']).lower() == 'none' or \
                        str(location['crawler']).lower() == 'false':
                    crawler = 'No'
                else:
                    crawler = 'Yes'

                if last24h_ip_count_dict.get(i.get('ip')) is None:
                    last24h_count = 0
                else:
                    last24h_count = attack_short_count(int(last24h_ip_count_dict.get(i.get('ip'))), 0)

                org = ''
                if len(str(location['org'])) > 1:
                    org = 'Org: ' + str(location['org']) + '\n'
                isp = ''
                if len(str(location['isp'])) > 1:
                    isp = 'ISP: ' + str(location['isp'])

                if location['cc'] in (None, 'none', 'None'):
                    if not is_local(i.get('ip')):
                        location['geo'] = _('Country not defined')
                    else:
                        location['geo'] = _('Private IP')
                else:
                    location['geo'] = "{0}, {1} ({2})".format(
                        str(location['city']),
                        str(location['country']),
                        str(location['cc']))

                ip_info_row.append([
                    i.get('ip'),
                    i.get('all_attacks'),
                    i.get('bt1'),
                    i.get('bt2'),
                    i.get('bt3'),
                    i.get('bt4'),
                    i.get('bt7'),
                    i.get('bt8'),
                    i.get('bt9'),
                    i.get('bt10'),
                    "Geo: " + location['geo'] + '\n' +
                    'Mobile: ' + mobile + '\n' +
                    'Proxy: ' + proxy + '\n' +
                    'Hosting: ' + hosting + '\n' +
                    'Crawler: ' + crawler + '\n' +
                    org + isp,
                    ' (' + str(last24h_count) + ' in 24 h.)',
                    str(location['cc']).lower()
                ])
            ip_info_json = json.dumps(ip_info_row)

            mcache.set(mcache_key, ip_info_json, 600)
            mcache.delete(semaphore_mcache_key)
            return ip_info_json

        main_base = main.filter(timestamp__gte=last_month, timestamp__lte=new_end_date).exclude(possible=True)

        today_summary_base = main_base.filter(timestamp__gte=piechart_end_date).exclude(bt=5)
        last24_summary_base = main_base.filter(timestamp__gte=yesterday_date, timestamp__lte=new_end_date).exclude(bt=5)
        last_week_summary_base = main_base.filter(timestamp__gte=last_week, timestamp__lte=new_end_date).exclude(bt=5)
        last_month_summary_base = main_base.filter(timestamp__gte=last_month, timestamp__lte=new_end_date).exclude(bt=5)

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        s = req_get.get('search', '').strip()
        if s:
            try:
                charts = search_filter(s, charts)
                today_summary_base = search_filter(s, today_summary_base)
                last24_summary_base = search_filter(s, last24_summary_base)
                last_week_summary_base = search_filter(s, last_week_summary_base)
                last_month_summary_base = search_filter(s, last_month_summary_base)
            except Exception:
                charts = Logs.objects.using('waf').none()
                today_summary_base = Logs.objects.using('waf').none()
                last24_summary_base = Logs.objects.using('waf').none()
                last_week_summary_base = Logs.objects.using('waf').none()
                last_month_summary_base = Logs.objects.using('waf').none()

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        charts = charts.values('ip', 'timestamp', 'vhost', 'alias')
        top10 = last24_summary_base

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        today_summary = today_summary_base.values('vhost', 'bt').annotate(count=Count('id')).filter(
            bt__in=[1, 2, 3, 4, 7, 8, 9, 10])
        today_summary_data = today_summary.values(all_attacks=Count('bt')).order_by('-all_attacks').annotate(
            sa=Count('bt', filter=Q(bt__in=[1, 2])),
            na=Count('bt', filter=Q(bt__in=[3, 8])),
            bf=Count('bt', filter=Q(bt=7)),
            sms=Count('bt', filter=Q(bt=9)),
            ddos=Count('bt', filter=Q(bt=10)),
            ca=Count('bt', filter=Q(bt=4)),
        )

        today_summary_list = list()

        last24_summary = last24_summary_base.values('vhost', 'bt').annotate(count=Count('id')).filter(
            bt__in=[1, 2, 3, 4, 7, 8, 9, 10])
        last24_summary_data = last24_summary.values(all_attacks=Count('bt')).order_by('-all_attacks').annotate(
            sa=Count('bt', filter=Q(bt__in=[1, 2])),
            na=Count('bt', filter=Q(bt__in=[3, 8])),
            bf=Count('bt', filter=Q(bt=7)),
            sms=Count('bt', filter=Q(bt=9)),
            ddos=Count('bt', filter=Q(bt=10)),
            ca=Count('bt', filter=Q(bt=4)),
        )

        last24_summary_list = list()

        last_week_summary = last_week_summary_base.values('vhost', 'bt').annotate(count=Count('id')).filter(
            bt__in=[1, 2, 3, 4, 7, 8, 9, 10])
        last_week_summary_data = last_week_summary.values(all_attacks=Count('bt')).order_by('-all_attacks').annotate(
            sa=Count('bt', filter=Q(bt__in=[1, 2])),
            na=Count('bt', filter=Q(bt__in=[3, 8])),
            bf=Count('bt', filter=Q(bt=7)),
            sms=Count('bt', filter=Q(bt=9)),
            ddos=Count('bt', filter=Q(bt=10)),
            ca=Count('bt', filter=Q(bt=4)),
        )

        last_week_summary_list = list()

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        last_month_summary = last_month_summary_base.values('vhost', 'bt').annotate(count=Count('id')). \
            filter(bt__in=[1, 2, 3, 4, 7, 8, 9, 10])
        last_month_summary_data = last_month_summary.values(all_attacks=Count('bt')).order_by('-all_attacks').annotate(
            sa=Count('bt', filter=Q(bt__in=[1, 2])),
            na=Count('bt', filter=Q(bt__in=[3, 8])),
            bf=Count('bt', filter=Q(bt=7)),
            sms=Count('bt', filter=Q(bt=9)),
            ddos=Count('bt', filter=Q(bt=10)),
            ca=Count('bt', filter=Q(bt=4)),
        )

        last_month_summary_list = list()

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        brute_force_summary_list = list()

        sms_flood_summary_list = list()

        ddos_summary_list = list()

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        internal_errors_summary_list = list()

        description_summary_list = list()

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        today_summary_json = json.dumps(today_summary_list)
        last24_summary_json = json.dumps(last24_summary_list)
        last_week_summary_json = json.dumps(last_week_summary_list)
        last_month_summary_json = json.dumps(last_month_summary_list)
        brute_force_summary_json = json.dumps(brute_force_summary_list)
        sms_flood_summary_json = json.dumps(sms_flood_summary_list)
        ddos_summary_json = json.dumps(ddos_summary_list)
        internal_errors_summary_json = json.dumps(internal_errors_summary_list)
        description_summary_json = json.dumps(description_summary_list)

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        linechart_data = charts \
            .annotate(date_line=Trunc('timestamp', 'day', output_field=DateTimeField())) \
            .values('date_line') \
            .order_by('date_line') \
            .annotate(all_attacks=Count('bt'), sa=Count('bt', filter=Q(bt__in=[1, 2])),
                      na=Count('bt', filter=Q(bt__in=[3, 8])), bf=Count('bt', filter=Q(bt=7)),
                      ddos=Count('bt', filter=Q(bt=10)), sms=Count('bt', filter=Q(bt=9)),
                      ca=Count('bt', filter=Q(bt=4)),
                      )

        rows2 = list()
        for i in linechart_data:
            rows2.append([
                i.get('date_line').strftime("%d.%m.%Y"),
                i.get('all_attacks'),
                i.get('sa'),
                i.get('na'),
                i.get('bf'),
                i.get('sms'),
                i.get('ddos'),
                i.get('ca'),
                0,
            ])

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        top10 = top10.values('vhost', 'bt', 'alias').annotate(count=Count('id')).filter(
            bt__in=[1, 2, 3, 4, 7, 8, 9, 10])
        host_top10_data = top10.values('vhost').annotate(
            all_attacks=Count('bt'),
            sa=Count('bt', filter=Q(bt__in=[1, 2])),
            na=Count('bt', filter=Q(bt__in=[3, 8])),
            bf=Count('bt', filter=Q(bt=7)),
            ca=Count('bt', filter=Q(bt=4)),
        ).order_by('-all_attacks')[:17]

        attackers_top10_data = top10.values('ip').annotate(
            all_attacks=Count('bt'),
            sa=Count('bt', filter=Q(bt__in=[1, 2])),
            na=Count('bt', filter=Q(bt__in=[3, 8])),
            bf=Count('bt', filter=Q(bt=7)),
            ca=Count('bt', filter=Q(bt=4)),
            sms=Count('bt', filter=Q(bt=9)),
            ddos=Count('bt', filter=Q(bt=10)),
        ).order_by('-all_attacks')[:10]

        ip_info_data = charts.values('ip').annotate(
            all_attacks=Count('bt'),
            bt1=Count('bt', filter=Q(bt=1)),
            bt2=Count('bt', filter=Q(bt=2)),
            bt3=Count('bt', filter=Q(bt=3)),
            bt4=Count('bt', filter=Q(bt=4)),
            bt7=Count('bt', filter=Q(bt=7)),
            bt8=Count('bt', filter=Q(bt=8)),
            bt9=Count('bt', filter=Q(bt=9)),
            bt10=Count('bt', filter=Q(bt=10)),
        ).order_by('-all_attacks')[:15]

        host_top10_row = list()
        for i in host_top10_data:
            host_top10_row.append([
                convert_from_punycode(i.get('vhost')),
                i.get('all_attacks'),
                i.get('sa'),
                i.get('na'),
                i.get('bf'),
                i.get('ca'),
            ])

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        ips_list = []
        for i in attackers_top10_data:
            ips_list.append(i.get('ip'))

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        for i in ip_info_data:
            ips_list.append(i.get('ip'))

        processes = []

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        lock = multiprocessing.Lock()
        for ip in list(chunks(list(set(ips_list)), 1))[:len(ips_list)]:
            proc = multiprocessing.Process(target=get_location_from_remote, args=(ip, mcache, lock))
            processes.append(proc)
            proc.start()

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        for pr in processes:
            pr.join(5)

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        last24h_ip_count_dict = dict()

        last24 = top10.values('ip').annotate(
            all_attacks=Count('bt'),
        ).order_by('-all_attacks')

        for i in last24:
            last24h_ip_count_dict[i.get('ip')] = str(i.get('all_attacks'))

        attackers_top10_row = list()

        attackers_top10_ip_info_dict = {}

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        for i in attackers_top10_data:
            if get_location_by_ip(mcache, i.get('ip')).get('status') != 'success':
                attackers_top10_ip_info_dict[i.get('ip')] = get_location_by_ip(mcache, i.get('ip'), fail=False)
            else:
                attackers_top10_ip_info_dict[i.get('ip')] = get_location_by_ip(mcache, i.get('ip'))

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        for i in attackers_top10_data:

            if str(attackers_top10_ip_info_dict[i.get('ip')]['mobile']).lower() == 'none' or \
                    str(attackers_top10_ip_info_dict[i.get('ip')]['mobile']).lower() == 'false':
                mobile = 'No'
            else:
                mobile = 'Yes'

            if str(attackers_top10_ip_info_dict[i.get('ip')]['proxy']).lower() == 'none' or \
                    str(attackers_top10_ip_info_dict[i.get('ip')]['proxy']).lower() == 'false':
                proxy = 'No'
            else:
                proxy = 'Yes'

            if str(attackers_top10_ip_info_dict[i.get('ip')]['hosting']).lower() == 'none' or \
                    str(attackers_top10_ip_info_dict[i.get('ip')]['hosting']).lower() == 'false':
                hosting = 'No'
            else:
                hosting = 'Yes'

            if str(attackers_top10_ip_info_dict[i.get('ip')]['crawler']).lower() == 'none' or \
                    str(attackers_top10_ip_info_dict[i.get('ip')]['crawler']).lower() == 'false':
                crawler = 'No'
            else:
                crawler = 'Yes'

            org = ''
            if len(str(attackers_top10_ip_info_dict[i.get('ip')]['org'])) > 1:
                org = 'Org: ' + str(attackers_top10_ip_info_dict[i.get('ip')]['org']) + '\n'
            isp = ''
            if len(str(attackers_top10_ip_info_dict[i.get('ip')]['isp'])) > 1:
                isp = 'ISP: ' + str(attackers_top10_ip_info_dict[i.get('ip')]['isp'])

            if attackers_top10_ip_info_dict[i.get('ip')]['cc'] in (None, 'none', 'None'):
                if not is_local(i.get('ip')):
                    attackers_top10_ip_info_dict[i.get('ip')]['geo'] = _('Country not defined')
                else:
                    attackers_top10_ip_info_dict[i.get('ip')]['geo'] = _('Private IP')
            else:
                attackers_top10_ip_info_dict[i.get('ip')]['geo'] = "{0}, {1} ({2})".format(
                    str(attackers_top10_ip_info_dict[i.get('ip')]['city']),
                    str(attackers_top10_ip_info_dict[i.get('ip')]['country']),
                    str(attackers_top10_ip_info_dict[i.get('ip')]['cc']))

            attackers_top10_row.append([
                i.get('ip'),
                i.get('all_attacks'),
                i.get('sa'),
                i.get('na'),
                i.get('bf'),
                i.get('ca'),
                i.get('ddos'),
                i.get('sms'),
                "Geo: " + attackers_top10_ip_info_dict[i.get('ip')]['geo'] + '\n' +
                'Mobile: ' + mobile + '\n' +
                'Proxy: ' + proxy + '\n' +
                'Hosting: ' + hosting + '\n' +
                'Crawler: ' + crawler + '\n' +
                org + isp,
                str(attackers_top10_ip_info_dict[i.get('ip')]['cc']).lower()
            ])
            ts = datetime.timestamp(datetime.now())
            mcache.set(semaphore_mcache_key, ts)

        ip_info_row = list()

        get_ip_info_dict = {}
        for i in ip_info_data:
            get_ip_info_dict[i.get('ip')] = get_location_by_ip(mcache, i.get('ip'))
            if get_ip_info_dict[i.get('ip')].get('status') != 'success':
                get_ip_info_dict[i.get('ip')] = get_location_by_ip(mcache, i.get('ip'), fail=False)

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        for i in ip_info_data:

            if str(get_ip_info_dict[i.get('ip')]['mobile']).lower() == 'none' or \
                    str(get_ip_info_dict[i.get('ip')]['mobile']).lower() == 'false':
                mobile = 'No'
            else:
                mobile = 'Yes'

            if str(get_ip_info_dict[i.get('ip')]['proxy']).lower() == 'none' or \
                    str(get_ip_info_dict[i.get('ip')]['proxy']).lower() == 'false':
                proxy = 'No'
            else:
                proxy = 'Yes'

            if str(get_ip_info_dict[i.get('ip')]['hosting']).lower() == 'none' or \
                    str(get_ip_info_dict[i.get('ip')]['hosting']).lower() == 'false':
                hosting = 'No'
            else:
                hosting = 'Yes'

            if str(get_ip_info_dict[i.get('ip')]['crawler']).lower() == 'none' or \
                    str(get_ip_info_dict[i.get('ip')]['crawler']).lower() == 'false':
                crawler = 'No'
            else:
                crawler = 'Yes'

            if last24h_ip_count_dict.get(i.get('ip')) is None:
                last24h_count = 0
            else:
                last24h_count = attack_short_count(int(last24h_ip_count_dict.get(i.get('ip'))), 0)

            org = ''
            if len(str(get_ip_info_dict[i.get('ip')]['org'])) > 1:
                org = 'Org: ' + str(get_ip_info_dict[i.get('ip')]['org']) + '\n'
            isp = ''
            if len(str(get_ip_info_dict[i.get('ip')]['isp'])) > 1:
                isp = 'ISP: ' + str(get_ip_info_dict[i.get('ip')]['isp'])

            if get_ip_info_dict[i.get('ip')]['cc'] in (None, 'none', 'None'):
                if not is_local(i.get('ip')):
                    get_ip_info_dict[i.get('ip')]['geo'] = _('Country not defined')
                else:
                    get_ip_info_dict[i.get('ip')]['geo'] = _('Private IP')
            else:
                get_ip_info_dict[i.get('ip')]['geo'] = "{0}, {1} ({2})".format(
                    str(get_ip_info_dict[i.get('ip')]['city']),
                    str(get_ip_info_dict[i.get('ip')]['country']),
                    str(get_ip_info_dict[i.get('ip')]['cc']))

            ip_info_row.append([
                i.get('ip'),
                i.get('all_attacks'),
                i.get('bt1'),
                i.get('bt2'),
                i.get('bt3'),
                i.get('bt4'),
                i.get('bt7'),
                i.get('bt8'),
                i.get('bt9'),
                i.get('bt10'),
                "Geo: " + get_ip_info_dict[i.get('ip')]['geo'] + '\n' +
                'Mobile: ' + mobile + '\n' +
                'Proxy: ' + proxy + '\n' +
                'Hosting: ' + hosting + '\n' +
                'Crawler: ' + crawler + '\n' +
                org + isp,
                '(' + str(last24h_count) + ' in 24 h.)',
                str(get_ip_info_dict[i.get('ip')]['cc']).lower()
            ])
            ts = datetime.timestamp(datetime.now())
            mcache.set(semaphore_mcache_key, ts)

        host_top10_json = json.dumps(host_top10_row)
        attackers_top10_json = json.dumps(attackers_top10_row)
        ip_info_json = json.dumps(ip_info_row)
        last_search_json = json.dumps(list(search_history_dict.keys())[0])
        linechart = json.dumps(rows2)

        res = str(linechart + ';' + host_top10_json + ";" + attackers_top10_json + ";" + today_summary_json + ";" +
                  last24_summary_json + ";" + last_week_summary_json + ";" + last_month_summary_json + ";" +
                  brute_force_summary_json + ";" + sms_flood_summary_json + ";" + internal_errors_summary_json + ";")

        if len(today_summary_data) >= 1:
            for row in today_summary_data:
                res += attack_short_count(row.get('all_attacks')) + ";" + attack_short_count(row.get('sa')) + ";" + \
                       attack_short_count(row.get('na')) + ";" + attack_short_count(row.get('bf')) + ";" + \
                       attack_short_count(row.get('sms')) + ";" + \
                       attack_short_count(row.get('ddos')) + ";" + attack_short_count(row.get('ca')) + ";"
        else:
            res += '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";"

        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts)

        if len(last24_summary_data) >= 1:
            for row in last24_summary_data:
                res += attack_short_count(row.get('all_attacks')) + ";" + attack_short_count(row.get('sa')) + ";" + \
                       attack_short_count(row.get('na')) + ";" + attack_short_count(row.get('bf')) + ";" + \
                       attack_short_count(row.get('sms')) + ";" + \
                       attack_short_count(row.get('ddos')) + ";" + attack_short_count(row.get('ca')) + ";"
        else:
            res += '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";"

        if len(last_week_summary_data) >= 1:
            for row in last_week_summary_data:
                res += attack_short_count(row.get('all_attacks')) + ";" + attack_short_count(row.get('sa')) + ";" + \
                       attack_short_count(row.get('na')) + ";" + attack_short_count(row.get('bf')) + ";" + \
                       attack_short_count(row.get('sms')) + ";" + \
                       attack_short_count(row.get('ddos')) + ";" + attack_short_count(row.get('ca')) + ";"
        else:
            res += '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";"

        if len(last_month_summary_data) >= 1:
            for row in last_month_summary_data:
                res += attack_short_count(row.get('all_attacks')) + ";" + attack_short_count(row.get('sa')) + ";" + \
                       attack_short_count(row.get('na')) + ";" + attack_short_count(row.get('bf')) + ";" + \
                       attack_short_count(row.get('sms')) + ";" + \
                       attack_short_count(row.get('ddos')) + ";" + attack_short_count(row.get('ca')) + ";"
        else:
            res += '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";" + '0' + ";"

        res += str(ip_info_json + ";" + description_summary_json + ";" + ddos_summary_json + ";" + last_search_json +
                   ";" + charts_history_json + ";")

        if s == '' and state == '':
            mcache.set(mcache_key, res, 3600)
        else:
            mcache.set(mcache_key, res, 3600)
        mcache.delete(semaphore_mcache_key)

        return res
    except Exception as e:
        logger.error('An error occurred during generation summary: {}'.format(str(e)))
        ts = datetime.timestamp(datetime.now())
        mcache.set(semaphore_mcache_key, ts - 45)
        return 0
