import json

from django.conf import settings
from django.shortcuts import render, redirect
from django.db import connections
from django.views.decorators.http import require_POST
from django.contrib.auth.decorators import login_required
from django.http import JsonResponse, HttpResponse
from django.contrib import messages
from django.utils.translation import pgettext, gettext as _

from main.decorators import time_of_function
from main.models.my_waf import LicKeys
from main.utils import convert_size, filter_upstreams, get_params, get_list_messages, convert_to_punycode, \
    convert_from_punycode, check_db
from my_waf.utils import get_waf_id, check_keys
from nginx.utils import get_nginx_json, set_nginx_conf
from nginx.utils import list_type_bool_keys, list_type_list_keys, list_type_int_keys, list_type_dynamic_keys, \
    list_type_float_keys

new_table_exist = check_db()

cache_ip, cache_port = settings.CACHES['default']['LOCATION'].split(':')
cache_address = (cache_ip, int(cache_port))

PAGE_SIZE = settings.DEFAULT_PAGE_SIZE

vGeoIP = settings.V_GEOIP
vAPP = settings.V_APP

isCCEnable = None
try:
    r = "SELECT * FROM information_schema.columns WHERE column_name = 'cc' AND table_name = 'attack'"
    with connections['waf'].cursor() as cursor:
        cursor.execute(r)
        res_cc = cursor.fetchone()
        if res_cc is None:
            isCCEnable = False
        else:
            isCCEnable = True

except Exception:
    print('Warning. Field "cc" is not exist in table "attack"')
    isCCEnable = False


def index_nginx(request):
    keys_list = LicKeys.objects.filter(user__id=request.user.id).all()
    context = {'keys_list': keys_list, 'home': settings.HOME_PAGE}
    return render(request, 'nginx/nginx_vhost.html', context)


def main_nginx(request):
    keys = LicKeys.objects.filter(user__id=request.user.id).all()
    keys_list = []
    lic_keys, checked_ids = check_keys(keys, check_waf_id=True)

    for key in lic_keys:
        waf_id = get_waf_id(key.key)
        if waf_id is not None:
            key.waf_id = waf_id
            keys_list.append(key)

    for key in keys_list:
        get_conf = get_nginx_json(key.key)
        print(get_conf)
        if get_conf:
            for i in get_conf:
                if i in list_type_list_keys:
                    get_conf[i] = ', '.join(get_conf[i])

            key.conf = get_conf

    context = {'keys_list': keys_list, 'home': settings.HOME_PAGE}
    return render(request, 'nginx/nginx_main.html', context)


@require_POST
@login_required
@time_of_function
def set_settings_conf(request):
    selected_key = request.POST.get('key')
    hashed_key = LicKeys.objects.get(id=selected_key).key
    data = request.POST.get('data')
    table = request.POST.get('table')
    try:
        data = json.loads(data)
    except json.JSONDecodeError:
        messages.error(request, pgettext('messages', "Invalid format"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)
        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    if table == 'nginx':
        r = set_nginx_conf(hashed_key, data)
        if not r[0]:
            messages.error(request, pgettext('messages', r[1]))
            storage = messages.get_messages(request)
            list_messages = get_list_messages(storage)

            return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    else:
        messages.error(request, pgettext('messages', "Invalid table"))
        storage = messages.get_messages(request)
        list_messages = get_list_messages(storage)

        return JsonResponse(data={'messages': list_messages, 'status': 'error'}, status=255)

    return JsonResponse(data={"status": "success"}, status=200)
