from dataclasses import dataclass

from django.conf import settings


@dataclass()
class AttackCounter:
    description: str
    numbers_of_attacks: int


def make_attack_type_counters(attacks_logs) -> list:
    """ Returns a list of objects that are needed to fill in the beginning of the report """
    list_by_descr = make_counter_list_by_description(attacks_logs)
    list_by_bt = make_counter_list_by_bt(attacks_logs)
    other = make_other_counter(attacks_logs)

    res = list_by_descr + list_by_bt
    res.append(other)
    return res


def make_other_counter(attacks_logs) -> AttackCounter:
    """ counting attacks not included in various categories """
    exclude_numbers = [bt for bt in settings.TITLE_BT_COUNTERS.keys()]
    exclude_descriptions = list(settings.TITLE_BT_DESCRIPTION_COUNTER.keys())
    other_count = attacks_logs.exclude(bt__in=exclude_numbers).exclude(
        description__iregex=r'(' + '|'.join(exclude_descriptions) + ')').count()
    return AttackCounter(
        description='other',
        numbers_of_attacks=other_count
    )


def make_counter_list_by_bt(attacks_logs) -> list:
    res = []
    bt_list = [bt for bt in settings.TITLE_BT_COUNTERS.keys()]
    for bt in bt_list:
        res.append(
            AttackCounter(
                numbers_of_attacks=attacks_logs.filter(bt=bt).count(),
                description=settings.TITLE_BT_COUNTERS[bt],
            )
        )
    return res


def make_counter_list_by_description(attacks_logs) -> list:
    res = []
    for db_description, report_description in settings.TITLE_BT_DESCRIPTION_COUNTER.items():
        res.append(
            AttackCounter(
                numbers_of_attacks=attacks_logs.filter(description__icontains=db_description).count(),
                description=report_description,
            )
        )
        return res


def make_extra_counters(attacks_logs) -> list:
    res = []
    for description, bt_list in settings.TITLE_BT_EXTRA_COUNTERS.items():
        res.append(
            AttackCounter(
                description=description,
                numbers_of_attacks=attacks_logs.filter(bt__in=bt_list).count()
            )

        )

    return res
