import threading
from datetime import datetime, timedelta

from django.conf import settings
from django.db.models import Count

from main.common import get_filtered_logs
from main.models import WAFUSER, Logs
from utils.http import get_country_by_ip


def get_start_datetime(request) -> datetime:
    if 'start' in request.POST.keys():
        return datetime.strptime(request.POST.get('start'), "%d.%m.%Y")
    else:
        return (datetime.now() + timedelta(days=-30)).replace(hour=00, minute=00, second=00, microsecond=0)


def get_end_datetime(request) -> datetime:
    if 'end' in request.POST.keys():
        return datetime.strptime(request.POST.get('end'), "%d.%m.%Y").replace(hour=23, minute=59, second=59)
    else:
        datetime.now()


def date_to_str(date: datetime) -> str:
    # format 12.01.2023
    return date.strftime("%d.%m.%Y")


def date_to_str_with_time(date: datetime) -> str:
    # format 14.12.23 15:09:50
    return date.strftime("%d.%m.%y %H:%M:%S")


def get_filtered_attacks(start_date, end_date, user_id):
    domain_from_wafuser = WAFUSER.objects.get(id=user_id).domain
    user_alias = WAFUSER.objects.get(id=user_id).waf_alias
    attacks_logs = Logs.objects.using('waf').filter(timestamp__gte=start_date, timestamp__lte=end_date)
    attacks_logs = get_filtered_logs(attacks_logs, user_alias, domain_from_wafuser)
    attacks_logs = attacks_logs.values(*settings.FIELDS_FOR_REPOST).exclude(
        bt__in=settings.BT_REPORT_EXCLUDE_NUMBERS).exclude(
        possible=True)
    return attacks_logs


def get_top_ip_list(attacks_logs) -> list:
    """ return list of dicts with ip, count, country"""
    top_ip = attacks_logs.values('ip').annotate(Count('ip')).order_by('-ip__count')[:20]
    top_ip = list(top_ip)
    _add_ip_countries(top_ip)
    return top_ip


def _add_ip_countries(top_ip: list):
    """ add ip country to every dict of list"""
    for ip_dict in top_ip:
        thr = threading.Thread(target=_get_country_by_ip_thread, args=(ip_dict,))
        thr.start()
        thr.join()


def _get_country_by_ip_thread(ip_dict: dict):
    ip_dict['country'] = get_country_by_ip(ip_dict['ip'])
