import requests
from django.conf import settings
from django.core.cache import cache

proxyDict = {
    "http": settings.HTTP_PROXY_CONF,
    "https": settings.HTTP_PROXY_CONF
}

headers = {'User-agent': settings.USER_AGENT}


def get_country_by_ip(ip_address: str) -> str:
    """ returns short form: FR, US, KZ etc"""
    cache_prefix = 'country_'
    country = cache.get(cache_prefix + ip_address)
    if country:
        return country

    url = f"{settings.URI_GEOIP}{ip_address}"
    response = requests.get(url=url,
                            proxies=proxyDict,
                            headers=headers,
                            timeout=settings.URI_GEOIP_TIMEOUT)
    if response.status_code == 200:
        data = response.json()
        if data.get("status") == "success":
            country = data.get("cc")
        else:
            country = data.get("description")
        cache.set(cache_prefix + ip_address, country, 24 * 60 * 60)
        return country
    else:
        return 'n/a'
